////////////////////////////////////////////////////////////////////////////////
// File         : HnetUniverse.h
// Subsystem    : Graph
// Component    : H-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class HnetUniverse
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 17.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// The class HnetUniverse implements the management of h-net types in a graph, 
// which has the structure of a tree.
// For detailed information see the document S-net.doc.
//     
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_HNETUNIVERSE_H__9A1F6C7D_BBB1_4D44_AC89_7B059B946884__INCLUDED_)
#define AFX_HNETUNIVERSE_H__9A1F6C7D_BBB1_4D44_AC89_7B059B946884__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Hnet.h"

class HnetUniverse  
{
public:
	HnetUniverse();
	HnetUniverse( HnetUniverse &universe );
	virtual ~HnetUniverse();
	HnetUniverse & operator=( const HnetUniverse &universe);
	IsoTypeNode CreateIsoType( int numOuterEntries = 0, int numOuterExits =0 );
	IsoTypeNode CreateIsoTypeElementary( int numOuterEntries, int numOuterExits );
	IsoTypeNode CopyIsoType( const IsoTypeNode &isoTypeOrig );
	void DelIsoType( IsoTypeNode &isoType );
	Hnet &GetHnet( const IsoTypeNode &isoType ) const { return m_isoTypeHnet[ isoType ]; };
	void AddReference( const IsoTypeNode &isoTypeFrom,  const IsoTypeNode &isoTypeTo );
	void DelReference( const IsoTypeNode &isoTypeFrom,  IsoTypeNode &isoTypeTo );
	IsoTypeNode Root( void ) const { return m_isoTypeGraph.FirstNode(); };
	bool ValidIsoType( IsoTypeNode &isoTypeNode ) const;

protected:
	Graph m_isoTypeGraph;
	NodeAttribute< Index > m_isoTypeReferenceCounter;

private:
	IsoTypeNode m_isoTypeRoot;
	mutable NodeAttribute< Hnet > m_isoTypeHnet;

protected:
	void Copy( const HnetUniverse &universe );

private:
	void DelIsoTypeRec( IsoTypeNode &isoType );

};

#endif // !defined(AFX_HNETUNIVERSE_H__9A1F6C7D_BBB1_4D44_AC89_7B059B946884__INCLUDED_)
