////////////////////////////////////////////////////////////////////////////////
// File         : HnetUniversePrint.h
// Subsystem    : Graph
// Component    : H-net
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class HnetUniversePrint
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 19.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// This class adds unique numbers to the nodes the graph of a h-net universe and
// allows to print out a h-net universe with a single call of Print().
//     
////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_HNETUNIVERSEPRINT_H__DE11DC16_526C_4B14_9067_36E1F6EA2477__INCLUDED_)
#define AFX_HNETUNIVERSEPRINT_H__DE11DC16_526C_4B14_9067_36E1F6EA2477__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ObjPrint.h"
#include "NodesNumbers.h"
#include "HnetUniverse.h"

class HnetUniversePrint : public ObjPrint, public NodesNumbers, public HnetUniverse  
{
public:
	HnetUniversePrint( const char *name = ObjPrintNoname );
	HnetUniversePrint( HnetUniversePrint &universe, const char *name = ObjPrintNoname);
	HnetUniversePrint( const HnetUniverse &universe, const char *name = ObjPrintNoname);
	virtual ~HnetUniversePrint();
	HnetUniversePrint& operator=( HnetUniversePrint &universe);
	NodeID NewIsoTypeID( const IsoTypeNode &isoTypeNode ) { return NodesNumbers::NewNodeID( isoTypeNode ); };
	NodeID GetIsoTypeID( const IsoTypeNode &isoTypeNode ) const { return NodesNumbers::GetNodeID( isoTypeNode ); };
	void SetIsoTypeID( const IsoTypeNode &isoTypeNode, NodeID id  ) { NodesNumbers::SetNodeID( isoTypeNode, id ); };
	void Renumber( void ) { NodesNumbers::Renumber( Root() ); };
	IsoTypeNode CreateIsoType( int numOuterEntries = 0, int numOuterExits = 0 );
	IsoTypeNode CreateIsoTypeElementary( int numOuterEntries, int numOuterExits );
	IsoTypeNode CopyIsoType( const IsoTypeNode &isoTypeOrig );
	void PrintIsoType( const IsoTypeNode &isoTypeNode ) ;
	void PrintIsoType( const NodeID &nodeID ) { PrintIsoType( GetIsoTypeNode( nodeID ) ); };
	void PrintIsoTypes( void ) const;

private:
	void Copy( HnetUniversePrint &universe);
	void PrintBody( void ) const; 
	void PrintNeighbours( const Node &n ) const;
	NodeID NewNodeID( const Node &n ) { return 0; }; // use NewIsoTypeID() instead
	NodeID GetNodeID( const Node &n ) { return 0; };  // use GetIsoTypeID() instead
	void SetNodeID( const Node &n, NodeID id ) {};  // use SetIsoTypeID() instead
	IsoTypeNode GetIsoTypeNode( NodeID nodeID );
};

#endif // !defined(AFX_HNETUNIVERSEPRINT_H__DE11DC16_526C_4B14_9067_36E1F6EA2477__INCLUDED_)
