////////////////////////////////////////////////////////////////////////////////
// File         : NodesNumbers.h
// Subsystem    : Graph
// Component    : trace objects
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class NodesNumbers
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 30.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
// 
// The class NodesNumbers simly manages a unique ID for each node,
// when it was created. For this it implement an integer node attribute.
// The class is used as base class for all objects that consists of nodes,
// for instance graphs, S-nets and H-nets.  
//     
////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_NODESNUMBERS_H__848C41A8_0EBA_42CD_B9E5_CD6BC87CC199__INCLUDED_)
#define AFX_NODESNUMBERS_H__848C41A8_0EBA_42CD_B9E5_CD6BC87CC199__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "graph.h"

typedef unsigned long NodeID;

const NodeID InvalidNodeID = 0xFFFF;

class NodesNumbers  
{
public:
	NodesNumbers() : m_currentNodeID( 0 ), m_nodeID( InvalidNodeID ){};
	virtual ~NodesNumbers() {};
	NodeID NewNodeID( const Node &n ) { m_nodeID[ n ] = m_currentNodeID; return m_currentNodeID++; };
	NodeID NodeIDinUse( void ) { return m_currentNodeID; };
	NodeID GetNodeID( const Node &n ) const { return m_nodeID[ n ]; };
	void SetNodeID( const Node &n, NodeID id );
	void Copy( const NodesNumbers &numb, const Node &n, const Node &nOrig );
	void Renumber( const Node &root );
	void RenumberNew( const Node &root );
	
private:
	NodeID m_currentNodeID;
	NodeAttribute< NodeID> m_nodeID;
};

#endif // !defined(AFX_NODESNUMBERS_H__848C41A8_0EBA_42CD_B9E5_CD6BC87CC199__INCLUDED_)
