#ifndef SEARCH_H
#define SEARCH_H

#include <deque>
#include "graph.h"

class GraphSearch
{
public:
	GraphSearch() : _g(0) {}
	void startDFS(const Graph &g, const Node &s=Node());
	void startDFSPost(const Graph &g, const Node &s=Node());
	void startBFS(const Graph &g, const Node &s=Node());
	void startBFSPost(const Graph &g, const Node &s=Node());
	void startBwdDFS(const Graph &g, const Node &t=Node());
	void startBwdDFSPost(const Graph &g, const Node &t=Node());
	void startBwdBFS(const Graph &g, const Node &t=Node());
	void startBwdBFSPost(const Graph &g, const Node &t=Node());
	bool done(void);
	void Next(void);	// returns at pre- xor post-hooks
	Node getCurrentNode(void);	// n, immer gltig
	Edge getCurrentEdge(void);	// e, myTargetNode(e)==n, bei Wurzelknoten ungltig!
	Node n;	// Alternative zu getCurrentNode()
	Edge e;	// Alternative zu getCurrentEdge()
	Edge getParentEdge(const Node &n);
private:
	void start(const Graph &g, const Node &s);
	Node myTargetNode(Edge &e);
	Edge myFirstIngoingEdge(Node &n);
	Edge myFirstOutgoingEdge(Node &n);
	Edge myNextOutgoingEdge(Edge &e);
	struct QueueEl
	{
		QueueEl(bool entering=false, Node n=Node(), Edge e=Edge()) : entering(entering), n(n), e(e) {}
		bool	entering;	// xor leaving
		Node	n;
		Edge	e;		// e.TargetNode() == n, auer bei Startknoten!
	};
	const Graph		*_g;			// == 0 -> done
	bool			dfs;			// xor bfs
	bool			preHook;		// xor postHook
	bool			fwd;			// xor bwd
	QueueEl			qe;			// head of queue
	std::deque<QueueEl>	queue;			// for iterative DFS/BFS
	NodeAttribute< bool >	visited;
	NodeAttribute< Edge >	firstVisitingEdge;	// defines tree
};

#endif // SEARCH_H

